/**
 * Enum for all inbuilt component types.
 * @see {@link ComponentType.CUSTOM} to create table columns with a custom svelte component
 */
export var ComponentType;
(function (ComponentType) {
    /**
     * Internal component type used when generating the component properties for default values
     *
     * @see {@link GenericComponentTypeProperties}
     */
    ComponentType["GENERIC"] = "generic";
    /**
     * General number type
     *
     * @see {@link NumberComponentTypeProperties}
     */
    ComponentType["NUMBER"] = "number";
    /**
     * General string type
     *
     * @see {@link StringComponentTypeProperties}
     */
    ComponentType["STRING"] = "string";
    /**
     * General boolean type with support for unusual boolean-like values, for example string values representing a boolean
     *
     * @see {@link BooleanComponentTypeProperties}
     */
    ComponentType["BOOLEAN"] = "boolean";
    /**
     * Enum type to display enum values fancier, translate them and have default/unknown value names.
     *
     * @see {@link EnumComponentTypeProperties}
     */
    ComponentType["ENUM"] = "enum";
    /**
     * Date type supporting the normal JavaScript Date as well as {@link https://moment.github.io/luxon luxon DateTime}
     *
     * @see {@link DateComponentTypeProperties}
     */
    ComponentType["DATE"] = "date";
    /**
     * Custom type to render a table column with a custom svelte component
     *
     * @see {@link CustomComponentTypeProperties}
     */
    ComponentType["CUSTOM"] = "custom";
})(ComponentType || (ComponentType = {}));
/**
 * Config for all inbuilt component types, containing the default value the type has, as well as default and forced type properties
 */
export const componentTypes = {
    generic: {
        defaultTypeProperties: {
            type: ComponentType.STRING,
            sortable: true,
            hidden: false
        }
    },
    boolean: {
        defaultValue: false,
        defaultTypeProperties: {
            truthy: true,
            inverted: false
        }
    },
    string: {
        defaultValue: ''
    },
    number: {
        defaultValue: 0
    },
    enum: {
        defaultValue: '',
        defaultTypeProperties: {
            enumColorKey: {
                default: 'dark',
                unknown: 'secondary'
            }
        }
    },
    custom: {
        defaultTypeProperties: {
            sort: false
        }
    },
    date: {
        defaultTypeProperties: {
            dateRelative: false,
            // Equivalent to luxons DATETIME_MED
            dateFormat: {
                year: 'numeric',
                month: 'short',
                day: 'numeric',
                hour: 'numeric',
                minute: 'numeric'
            }
        }
    }
};
