import { QueryResult } from './QueryResult.js';
import { hasOwnProperty } from '../util/generalUtil.js';
import { AbstractDataSource } from './AbstractDataSource.svelte.js';
/**
 * Uses the {@link https://developer.mozilla.org/en-US/docs/Web/API/Fetch_API Fetch API} to request the data from an api endpoint via HTTP POST requests
 */
export class FetchApiDataSource extends AbstractDataSource {
    url;
    options;
    /**
     * Create a new data source to fetch your paginated table data from an api endpoint.
     *
     * The endpoint will receive HTTP POST requests with a json body consisting of an {@link PaginatedListRequest}.
     * Your server has to answer with a {@link PaginatedListResponse} json body.
     * @param url a url to your api endpoint, for example `/api/users/list`
     * @param options an object with [request init options](https://developer.mozilla.org/en-US/docs/Web/API/fetch#options) which get passed directly to fetch
     */
    constructor(url, options = {}) {
        super();
        this.url = url;
        this.options = options;
    }
    requestData(data) {
        fetch(this.url, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(data),
            ...this.options
        })
            .then(async (response) => {
            let resData = undefined;
            try {
                resData = await response.json();
            }
            catch (error) {
                this.queryResult = QueryResult.buildError(error);
                return;
            }
            if (response.ok && typeof resData !== 'undefined') {
                this.queryResult = QueryResult.buildSuccess(resData);
            }
            else {
                const errorMessage = hasOwnProperty(resData, 'message')
                    ? String(resData.message)
                    : 'Unknown network error';
                const error = new Error(errorMessage);
                if (resData) {
                    Object.assign(error, resData);
                }
                this.queryResult = QueryResult.buildError(error);
            }
        })
            .catch((error) => {
            this.queryResult = QueryResult.buildError(error);
        });
    }
}
