import { QueryResult } from './QueryResult.js';
import { AbstractDataSource } from './AbstractDataSource.svelte.js';
/**
 * The local data source requires the data to be passed in as an array.
 * The dataTable is still paginated, sorted and searchable by performing everything on the clientside
 */
export class LocalDataSource extends AbstractDataSource {
    data;
    options;
    /**
     * Create a new local data source to use the passed in array as data.
     *
     * @param data array with all table entries
     * @param options
     */
    constructor(data, options = {}) {
        super();
        this.data = data;
        this.options = options;
    }
    requestData(data) {
        const { start, amount, orderBy, searchQuery } = data;
        const filteredData = this.filterData({
            data: this.data,
            searchQuery
        });
        const sortedData = this.sortData({
            data: filteredData,
            orderBy
        });
        const paginatedData = sortedData.slice(start, start + amount);
        this.queryResult = QueryResult.buildSuccess({
            totalCount: filteredData.length,
            items: paginatedData
        });
    }
    filterData({ data, searchQuery }) {
        if (!searchQuery) {
            return data;
        }
        if (this.options.filtering?.textSearchColumns?.length && searchQuery.searchText?.trim()) {
            data = data.filter((item) => {
                for (const columnKey of this.options.filtering.textSearchColumns) {
                    if (String(item[columnKey])
                        .toLowerCase()
                        .indexOf(searchQuery.searchText.toLowerCase()) !== -1) {
                        return true;
                    }
                }
                return false;
            });
        }
        if (this.options.filtering?.filterFunction) {
            data = data.filter((item) => this.options.filtering.filterFunction(item, searchQuery));
        }
        return data;
    }
    sortData({ data, orderBy }) {
        if (!orderBy) {
            return data;
        }
        const key = orderBy.column;
        const exampleEntry = data.find((item) => typeof item[key] !== 'undefined' && item[key] !== null);
        const exampleValue = exampleEntry ? String(exampleEntry[key]) : '';
        const isNumber = !isNaN(exampleValue) && !isNaN(parseFloat(exampleValue));
        if (isNumber) {
            return data.sort((a, b) => {
                const n1 = parseFloat(String(a[key]));
                const n2 = parseFloat(String(b[key]));
                return orderBy.order === 'asc' ? n1 - n2 : n2 - n1;
            });
        }
        else {
            return data.sort((a, b) => {
                const s1 = String(a[key]);
                const s2 = String(b[key]);
                return orderBy.order === 'asc' ? s1.localeCompare(s2) : s2.localeCompare(s1);
            });
        }
    }
}
