import { QueryResult } from '../QueryResult.js';
import { createQuery } from '@tanstack/svelte-query';
/**
 * Uses the Svelte Query library to fetch your paginated table data.
 */
export class SvelteQueryDataSource {
    apiFunction = $state();
    additionalQueryOptions = $state({});
    dataTableConfig = $state();
    queryKey = $state(undefined);
    queryOptions = $derived(this.createQueryOptions());
    dataQuery = createQuery(() => this.queryOptions);
    initialized = false;
    queryKeyPrefix = $state();
    queryEnabled = $state(true);
    /**
     * Create a new data source to fetch your paginated table data from an api endpoint
     * using Svelte Query in the background to manage requests, caching and more.
     *
     * @param apiFunction the api function which actually fetches your data, passed through to Svelte Query as the [query function](https://sveltequery.vercel.app/guides/query-functions).
     * A query function can be literally any function that returns a promise. The promise that is returned should either resolve the data or throw an error.
     * @param additionalQueryOptions additional options passed to Svelte Query
     *
     * @see {@link https://sveltequery.vercel.app/reference/useQuery}
     */
    constructor(apiFunction, additionalQueryOptions = {}) {
        this.apiFunction = apiFunction;
        this.additionalQueryOptions = additionalQueryOptions;
    }
    _queryResult = $derived(this.deriveQueryResult());
    get queryResult() {
        return this._queryResult;
    }
    setConfig(config) {
        this.dataTableConfig = config;
        this.queryKeyPrefix = `dataTable-${config.type}`;
        this.dataQuery = createQuery(() => this.queryOptions);
        this.initialized = true;
    }
    requestData(data) {
        if (!this.initialized || !this.queryKeyPrefix) {
            throw new Error('Svelte-Query data source was not properly initialized before requesting data');
        }
        this.queryEnabled ??= true;
        this.queryKey = [this.queryKeyPrefix, this.normalizeRequestData(data)];
    }
    createQueryOptions() {
        if (!this.queryKey || !this.apiFunction) {
            return {
                queryKey: [],
                enabled: false
            };
        }
        return {
            staleTime: 30 * 1000,
            gcTime: 60 * 1000,
            ...this.additionalQueryOptions,
            queryKey: this.queryKey,
            queryFn: this.wrapApiFunction()
        };
    }
    deriveQueryResult() {
        if (this.dataQuery.isSuccess) {
            return QueryResult.buildSuccess(this.dataQuery.data);
        }
        else if (this.dataQuery.isError) {
            return QueryResult.buildError(this.dataQuery.error);
        }
        else {
            return QueryResult.buildLoading();
        }
    }
    wrapApiFunction() {
        return ({ queryKey }) => {
            console.log('calling api with', $state.snapshot(queryKey[1]));
            return this.apiFunction(queryKey[1]);
        };
    }
    normalizeRequestData(data) {
        return {
            start: data.start ?? 0,
            amount: data.amount ?? 10,
            orderBy: data.orderBy && data.orderBy.column
                ? {
                    order: data.orderBy.order ?? 'desc',
                    column: data.orderBy.column
                }
                : undefined,
            rawSearchQuery: data.rawSearchQuery ?? '',
            searchQuery: {
                searchText: data.searchQuery?.searchText ?? '',
                searchCategories: data.searchQuery?.searchCategories ?? [],
                searchFilters: data.searchQuery?.searchFilters ?? [],
                forceGlobalSearch: data.searchQuery?.forceGlobalSearch ?? false
            }
        };
    }
}
