import { AbstractSearchParser } from './AbstractSearchParser.js';
import { SearchError } from './SearchError.js';
export class AdvancedSearchParser extends AbstractSearchParser {
    searchCategoryAliases;
    searchFilterAliases = {};
    additionalQueryPartParsers = [];
    enableCategoryParsing;
    enableFilterParsing;
    constructor(options) {
        super();
        this.searchCategoryAliases = options?.searchCategoryAliases ?? {};
        this.searchFilterAliases = options?.searchFilterAliases ?? {};
        this.additionalQueryPartParsers = options?.additionalQueryPartParsers ?? [];
        this.enableCategoryParsing = options?.enableCategoryParsing ?? true;
        this.enableFilterParsing = options?.enableFilterParsing ?? true;
    }
    parseSearchQuery(rawSearchQuery) {
        const queryParts = this.splitQueryParts(this.cleanQuery(rawSearchQuery));
        if (!queryParts.length) {
            return undefined;
        }
        const searchText = [];
        const searchCategories = [];
        const searchFilters = [];
        let forceGlobalSearch = false;
        for (let queryPart of queryParts) {
            if (this.enableCategoryParsing && queryPart.startsWith('#')) {
                queryPart = queryPart.substring(1);
                if (queryPart === '#') {
                    forceGlobalSearch = true;
                }
                else {
                    searchCategories.push(this.matchPartialAlias(this.searchCategoryAliases, queryPart));
                }
            }
            else if (this.enableFilterParsing && queryPart.indexOf(':') !== -1) {
                const filterName = queryPart.substring(0, queryPart.indexOf(':'));
                const filterArgs = queryPart.substring(queryPart.indexOf(':') + 1);
                if (!filterName.length) {
                    throw new SearchError('search.error.emptyFilterName', queryPart);
                }
                const filterType = this.matchPartialAlias(this.searchFilterAliases, filterName);
                searchFilters.push({
                    type: filterType,
                    value: filterArgs
                });
            }
            else {
                let parsedQueryPart = false;
                for (const queryPartParser of this.additionalQueryPartParsers) {
                    parsedQueryPart = queryPartParser(queryPart);
                    if (parsedQueryPart) {
                        break;
                    }
                }
                if (parsedQueryPart) {
                    if (parsedQueryPart.searchFilter) {
                        searchFilters.push(parsedQueryPart.searchFilter);
                    }
                    if (parsedQueryPart.searchCategory) {
                        searchCategories.push(parsedQueryPart.searchCategory);
                    }
                }
                else {
                    searchText.push(queryPart);
                }
            }
        }
        return {
            searchText: searchText.join(' '),
            searchCategories,
            searchFilters,
            forceGlobalSearch
        };
    }
    matchPartialAlias(aliasMap, text) {
        let match = null;
        for (const key of Object.keys(aliasMap)) {
            for (const alias of [key, ...aliasMap[key]]) {
                if (alias.startsWith(text)) {
                    if (match && match !== key) {
                        throw new SearchError('search.error.unspecificAlias', match, alias);
                    }
                    match = key;
                }
            }
        }
        if (!match) {
            // throw new SearchError('search.error.unknownSpecifier', text);
        }
        return match ?? text;
    }
}
